﻿using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Collections.Specialized;
using OpcLabs.BaseLib.OperationModel;

namespace OpcLabs.BaseLibTest.OperationModel
{
    public partial class OperationResultTest
    {
        #region Test Methods
        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void Formatting_F_Failure_ErrorCode_Test()
        {
            Assert.AreEqual("*** Failure: -2 (0xFFFFFFFE): Some error [...] (0 diagnostics)", new OperationResult { ErrorMessage = "Some error\nSecond line", ErrorCode = -2 }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Failure_NoErrorCode_Test()
        {
            Assert.AreEqual("*** Failure: 0 (0x00000000): Some error [...] (0 diagnostics)", new OperationResult { ErrorMessage = "Some error\nSecond line" }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success 0 (0x00000000):  (4 diagnostics)",
                new OperationResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList())
                }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Success_ErrorCode_Test()
        {
            Assert.AreEqual("Success 3 (0x00000003):  (0 diagnostics)", new OperationResult { ErrorCode = 3 }.ToString("F"));
        }

        [TestMethod]
        public void Formatting_F_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual("Success 0 (0x00000000):  (0 diagnostics)", new OperationResult().ToString("F"));
        }

        [TestMethod]
        public void Formatting_g_Failure_ErrorCode_Test()
        {
            Assert.AreEqual("*** Failure -2: Some error [...]", new OperationResult { ErrorMessage = "Some error\nSecond line", ErrorCode = -2 }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Failure_NoErrorCode_Test()
        {
            Assert.AreEqual("*** Failure: Some error [...]", new OperationResult { ErrorMessage = "Some error\nSecond line" }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success",
                new OperationResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList())
                }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Success_ErrorCode_Test()
        {
            Assert.AreEqual("Success 3", new OperationResult { ErrorCode = 3 }.ToString("g"));
        }

        [TestMethod]
        public void Formatting_g_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual("Success", new OperationResult().ToString("g"));
        }

        [TestMethod]
        public void ToString_Failure_ErrorCode_Test()
        {
            Assert.AreEqual("*** Failure -2 (0xFFFFFFFE): Some error [...]", new OperationResult { ErrorMessage = "Some error\nSecond line", ErrorCode = -2 }.ToString());
        }

        [TestMethod]
        public void ToString_Failure_NoErrorCode_Test()
        {
            Assert.AreEqual("*** Failure: Some error [...]", new OperationResult { ErrorMessage = "Some error\nSecond line" }.ToString());
        }

        [TestMethod]
        public void ToString_Success_Diagnostics_Test()
        {
            Assert.AreEqual(
                "Success (4 diagnostics)",
                new OperationResult
                {
                    Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList())
                }.ToString());
        }

        [TestMethod]
        public void ToString_Success_ErrorCode_Test()
        {
            Assert.AreEqual("Success 3 (0x00000003)", new OperationResult { ErrorCode = 3 }.ToString());
        }

        [TestMethod]
        public void ToString_Success_NoDiagnostics_Test()
        {
            Assert.AreEqual("Success", new OperationResult().ToString());
        }

        // ReSharper restore InconsistentNaming
        #endregion
    }
}
