﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLibFactory.Text;
using OpcLabs.UnitTestLib;

namespace OpcLabs.BaseLibTest.Text
{
    public partial class MacroExpanderTest
    {
        #region Test Methods
        // ReSharper disable InconsistentNaming

        [TestMethod]
        public void ExpandStringTest_1_TemplateString1()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(TemplateString1);
            Assert.AreEqual(ExpandedString1, actual);
        }

        [TestMethod]
        public void ExpandStringTest_2_TemplateString1()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(TemplateString1, out IEnumerable<string> errors);
            Assert.AreEqual(ExpandedString1, actual);
            Assert.IsFalse(errors.Any());
        }

        [TestMethod]
        public void ExpandStringTest_3_TemplateString1()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(TemplateString1, out int errorCount);
            Assert.AreEqual(ExpandedString1, actual);
            Assert.AreEqual(0, errorCount);
        }

        [TestMethod]
        public void ExpandStringTest_1_TemplateString2()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(TemplateString2);
            Assert.AreEqual(ExpandedString2, actual);
        }

        [TestMethod]
        public void ExpandStringTest_2_TemplateString2()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(TemplateString2, out IEnumerable<string> errors);
            Assert.AreEqual(ExpandedString2, actual);
            Assert.IsFalse(errors.Any());
        }

        [TestMethod]
        public void ExpandStringTest_3_TemplateString2()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(TemplateString2, out int errorCount);
            Assert.AreEqual(ExpandedString2, actual);
            Assert.AreEqual(0, errorCount);
        }

        [TestMethod]
        public void ExpandStringTest_1_TemplateString3()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(TemplateString3);
            Assert.AreEqual(ExpandedString3, actual);
        }

        [TestMethod]
        public void ExpandStringTest_2_TemplateString3()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(TemplateString3, out IEnumerable<string> errors);
            Assert.AreEqual(ExpandedString3, actual);
            Assert.IsTrue(errors.SequenceEqual(new[] {"unknownMacro"}));
        }

        [TestMethod]
        public void ExpandStringTest_3_TemplateString3()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(TemplateString3, out int errorCount);
            Assert.AreEqual(ExpandedString3, actual);
            Assert.AreEqual(1, errorCount);
        }

        [TestMethod]
        public void ExpandStringTest_1_TemplateStringEmpty()
        {
            var target = MacroExpanderInstances.Empty;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(String.Empty);
            Assert.AreEqual(String.Empty, actual);
        }

        [TestMethod]
        public void ExpandStringTest_2_TemplateStringEmpty()
        {
            var target = MacroExpanderInstances.Empty;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(String.Empty, out IEnumerable<string> errors);
            Assert.AreEqual(String.Empty, actual);
            Assert.IsFalse(errors.Any());
        }

        [TestMethod]
        public void ExpandStringTest_3_TemplateStringEmpty()
        {
            var target = MacroExpanderInstances.Empty;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(String.Empty, out int errorCount);
            Assert.AreEqual(String.Empty, actual);
            Assert.AreEqual(0, errorCount);
        }

        [TestMethod]
        public void ExpandStringTest_3_Instance1_TemplateStringEmpty()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            string actual = target.ExpandString(String.Empty, out int errorCount);
            Assert.AreEqual(String.Empty, actual);
            Assert.AreEqual(0, errorCount);
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestMethod]
        public void ExpandStringTest_2_TemplateNull()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            target.ExpandString(
                // ReSharper disable once AssignNullToNotNullAttribute
                null,
                out IEnumerable<string> errors);
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestMethod]
        public void ExpandStringTest_3_TemplateNull()
        {
            var target = MacroExpanderInstances.Instance1;
            AssertUtilities.WriteLine(target);
            target.ExpandString(
                // ReSharper disable once AssignNullToNotNullAttribute
                null,
                out int errorCount);
        }

        // ReSharper restore InconsistentNaming
        #endregion



        #region Private Constants

        private const string ExpandedString1 = "AAAVALUE3BBBvAlUe2CCCValue1DDD";
        private const string ExpandedString2 = ExpandedString1;
        private const string ExpandedString3 = "AAAVALUE3BBB??<unknownMacro>CCCValue1DDD";

        private const string TemplateString1 = "AAA$(MACRO3)BBB$(mAcRo2)CCC$(Macro1)DDD";
        private const string TemplateString2 = "AAA$(Macro3)BBB$(MaCrO2)CCC$(MACRO1)DDD";
        private const string TemplateString3 = "AAA$(MACRO3)BBB$(unknownMacro)CCC$(Macro1)DDD";

        #endregion

    }
}
