﻿
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.ComInterop;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpcLiveTest.DataAccess.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.DataAccess
{
    partial class EasyDAClientTest_Xml
    {
        [DynamicData(nameof(OpcFoundation_XmlDaSampleServer.CommonPropertyCatalogEntryDataRows), typeof(OpcFoundation_XmlDaSampleServer))]
        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void GetPropertyValueCatalogTest_Xml_Basic([NotNull] DAPropertyCatalogEntry propertyCatalogEntry)
        {
            DANodeDescriptor nodeDescriptor = new DANodeDescriptor("Static/Analog Types/Int") { NodePath = "" };
            GetPropertyValueCatalogTest_Basic(
                TestEnvironment.OpcFoundation_XmlDaSampleServer.ServerDescriptors.Http, 
                nodeDescriptor,
                propertyCatalogEntry);
        }

        // The server returns E_WRITEONLY when getting the Timestamp property of a write-only item, so we need to use
        // readable items only.
        //[DynamicData(nameof(OpcFoundation_XmlDaSampleServer.AllItemCatalogEntryDataRows), typeof(OpcFoundation_XmlDaSampleServer))]
        [DynamicData(nameof(OpcFoundation_XmlDaSampleServer.ReadableItemCatalogEntryDataRows), typeof(OpcFoundation_XmlDaSampleServer))]
        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void GetPropertyValueCatalogTest_Xml_Timestamp([NotNull] DANodeCatalogEntry nodeCatalogEntry)
        {
            GetPropertyValueCatalogTest_Timestamp(
                TestEnvironment.OpcFoundation_XmlDaSampleServer.ServerDescriptors.Http, 
                nodeCatalogEntry);
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void GetPropertyValueTest_Xml_QualifiedName()
        {
            TestSection.Arrange();
            ServerDescriptor serverDescriptor = TestEnvironment.OpcFoundation_XmlDaSampleServer.ServerDescriptors.Http;
            TestEnvironment.PrecheckServer(serverDescriptor);

            var target = new EasyDAClient();
            DANodeDescriptor nodeDescriptor = new DANodeDescriptor("Static/Analog Types/Int") { NodePath = "" };

            TestSection.Act();
            object actualValue = target.GetPropertyValue(serverDescriptor, nodeDescriptor, "dataType");

            TestSection.Assert();
            AssertUtilities.WriteLine(actualValue);
            Assert.IsNotNull(actualValue);
            Assert.AreEqual((short)VarTypes.I4, actualValue);
        }
    }
}
