﻿
using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpcLiveTest.DataAccess.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.DataAccess
{
    partial class EasyDAClientTest_Xml
    {
        [DynamicData(nameof(OpcFoundation_XmlDaSampleServer.ReadableItemCatalogEntryDataRows), typeof(OpcFoundation_XmlDaSampleServer))]
        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadItemCatalogTest_Xml_Basic([NotNull] DAItemCatalogEntry itemCatalogEntry)
        {
            ReadItemCatalogTest_Basic(TestEnvironment.OpcFoundation_XmlDaSampleServer.ServerDescriptors.Http, itemCatalogEntry);
        }

        [DynamicData(nameof(OpcFoundation_XmlDaSampleServer.NonReadableItemCatalogEntryDataRows), typeof(OpcFoundation_XmlDaSampleServer))]
        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadItemCatalogTest_Xml_NonReadable([NotNull] DAItemCatalogEntry itemCatalogEntry)
        {
            ReadItemTest_NonReadable(TestEnvironment.OpcFoundation_XmlDaSampleServer.ServerDescriptors.Http, itemCatalogEntry.Element);
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadItemValueTest_Xml_Int()
        {
            TestSection.Arrange();
            ServerDescriptor serverDescriptor = TestEnvironment.OpcFoundation_XmlDaSampleServer.ServerDescriptors.Http;
            TestEnvironment.PrecheckServer(serverDescriptor);

            var target = new EasyDAClient();
            DAItemDescriptor itemDescriptor = "Static/Analog Types/Int";

            TestSection.Act();
            object actualValue = target.ReadItemValue(serverDescriptor, itemDescriptor);

            TestSection.Assert();
            AssertUtilities.WriteLine(actualValue);
            Assert.IsNotNull(actualValue);
            Assert.IsInstanceOfType(actualValue, typeof(Int32));
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadItemValueTest_Xml_IntArray()
        {
            TestSection.Arrange();
            ServerDescriptor serverDescriptor = TestEnvironment.OpcFoundation_XmlDaSampleServer.ServerDescriptors.Http;
            TestEnvironment.PrecheckServer(serverDescriptor);

            var target = new EasyDAClient();
            DAItemDescriptor itemDescriptor = "Static/Analog Types/Int[]";

            TestSection.Act();
            object actualValue = target.ReadItemValue(serverDescriptor, itemDescriptor);

            TestSection.Assert();
            AssertUtilities.WriteLine(actualValue);
            Assert.IsNotNull(actualValue);
            Assert.IsInstanceOfType(actualValue, typeof(Int32[]));
            var typedValue = (Int32[])actualValue;
            Assert.AreEqual(0, typedValue.GetLowerBound(0));
            int length = typedValue.Length;
            AssertUtilities.WriteLine(length);
            for (int i = 0; i < length; i++)
                AssertUtilities.WriteLine($"  {i}: {typedValue[i]}");
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadItemValueTest_Xml_OpcXmlDa()
        {
            TestSection.Arrange();
            ServerDescriptor serverDescriptor = TestEnvironment.OpcFoundation_XmlDaSampleServer.ServerDescriptors.Http;
            serverDescriptor.Scheme = "opc.xmlda";
            TestEnvironment.PrecheckServer(serverDescriptor);

            var target = new EasyDAClient();
            DAItemDescriptor itemDescriptor = "Static/Analog Types/Int";

            TestSection.Act();
            object actualValue = target.ReadItemValue(serverDescriptor, itemDescriptor);

            TestSection.Assert();
            AssertUtilities.WriteLine(actualValue);
            Assert.IsNotNull(actualValue);
            Assert.IsInstanceOfType(actualValue, typeof(Int32));
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadMultipleItemsCatalogTest_Xml()
        {
            ReadMultipleItemsCatalogTest(TestEnvironment.OpcFoundation_XmlDaSampleServer.ServerDescriptors.Http, OpcFoundation_XmlDaSampleServer.AllItemCatalogEntries);
        }
    }
}
