﻿using System;
using System.Globalization;
using System.Threading;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.DataAccess
{
    // ReSharper disable once InconsistentNaming
    public partial class IEasyDAClientTest_Com
    {
        #region Test Methods
        // ReSharper disable InconsistentNaming

        /// <summary>
        /// This test reads different items in a loop.
        /// </summary>
        /// <remarks>There is no particular made-up reason for testing this, but the test is meant as part of load test,
        /// as we had a customer scenario where similar setup (in Web app under IIS) resulted in a crash when processing
        /// multiple client request at once.</remarks>
        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadItemTest_RepeatedWithDifferentItems_TightLoop()
        {
            var target = CreateClient();
            for (int i = 0; i < 8; i++)
            {
                string itemId = String.Format(CultureInfo.InvariantCulture, "Simulation.Incrementing.Copy_1.Phase_{0}", i + 1);
                DAVtq actualVtq = target.ReadItem("", LegacyTestDefs.ProgIds.KitServer, itemId);

                Assert.IsNotNull(actualVtq);
                Assert.IsTrue(actualVtq.Quality.IsGood);
                Assert.IsTrue(actualVtq.Quality.StatusBitField == DAStatusChoice.GoodNonspecific);
                Assert.IsTrue(actualVtq.Quality.LimitBitField == DALimitChoice.LimitOk);
                Assert.IsTrue(actualVtq.HasValue);
                //... test more
            }
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadItemTest_Repeated_TightLoop()
        {
            const int repeatCount = 1000;
            var target = CreateClient();
            for (int i = 0; i < repeatCount; i++)
            {
                DAVtq actualVtq = target.ReadItem("", LegacyTestDefs.ProgIds.KitServer, "Simulation.Random");

                Assert.IsNotNull(actualVtq);
                Assert.IsTrue(actualVtq.Quality.IsGood);
                Assert.IsTrue(actualVtq.Quality.StatusBitField == DAStatusChoice.GoodNonspecific);
                Assert.IsTrue(actualVtq.Quality.LimitBitField == DALimitChoice.LimitOk);
                Assert.IsTrue(actualVtq.HasValue);
                //... test more

                AssertUtilities.WriteLine(String.Format(CultureInfo.InvariantCulture,
                    "{0}: {1}", System.Environment.TickCount, actualVtq.Value));
            }
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void ReadItemTest_Repeated_100ms_ByValueAge_1000ms()
        {
            ServerDescriptor serverDescriptor = LegacyTestDefs.ProgIds.KitServer;
            //ServerDescriptor serverDescriptor = "OpcTestLab.OpcAnalyzer.1";

            var target = CreateClient();
            //target.UpdateRates.ReadAutomatic = Timeout.Infinite;
            const int repeatCount = 100;    // IMPROVE: have long-running test with more
            for (int i = 0; i < repeatCount; i++)
            {
                AssertUtilities.WriteLine(i);

                DAVtq actualVtq = target.ReadItem(serverDescriptor, "Simulation.Random",
                    new DAReadParameters(DADataSource.ByValueAge, 1000));

                Assert.IsNotNull(actualVtq);
                Assert.IsTrue(actualVtq.Quality.IsGood);
                Assert.IsTrue(actualVtq.Quality.StatusBitField == DAStatusChoice.GoodNonspecific);
                Assert.IsTrue(actualVtq.Quality.LimitBitField == DALimitChoice.LimitOk);
                Assert.IsTrue(actualVtq.HasValue);
                //... test more

                AssertUtilities.WriteLine(actualVtq.Value);
                AssertUtilities.WaitingPhase(100);
            }
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void ReadItemTest_Repeated_100ms_Cache()
        {
            const int repeatCount = 100;    // IMPROVE: have long-running test with more
            var target = CreateClient();
            for (int i = 0; i < repeatCount; i++)
            {
                DAVtq actualVtq = target.ReadItem(LegacyTestDefs.ProgIds.KitServer, "Simulation.Random",
                    new DAReadParameters(DADataSource.Cache));

                Assert.IsNotNull(actualVtq);
                Assert.IsTrue(actualVtq.Quality.IsGood);
                Assert.IsTrue(actualVtq.Quality.StatusBitField == DAStatusChoice.GoodNonspecific);
                Assert.IsTrue(actualVtq.Quality.LimitBitField == DALimitChoice.LimitOk);
                Assert.IsTrue(actualVtq.HasValue);
                //... test more

                AssertUtilities.WriteLine(String.Format(CultureInfo.InvariantCulture,
                    "{0}: {1}", System.Environment.TickCount, actualVtq.Value));
                AssertUtilities.WaitingPhase(100);
            }
        }

        [Ignore]    // IMPROVE: put back; is long-running
        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void ReadItemTest_Repeated_VaryingDelay()
        {
            ServerDescriptor serverDescriptor = LegacyTestDefs.ProgIds.KitServer;
            //ServerDescriptor serverDescriptor = "OpcTestLab.OpcAnalyzer.1";

            //EasyDAClient.SharedParameters.Topic.SlowdownWeight = 0.0f;
            //EasyDAClient.SharedParameters.Topic.SpeedupWeight = 0.0f;
            var target = CreateClient();
            //target.InstanceParameters.UpdateRates.ReadAutomatic = Timeout.Infinite;
            //target.InstanceParameters.UpdateRates.WriteAutomatic = Timeout.Infinite;

            var random = new Random();
            int delay = 0;
            for (int i = 0; i < 200; i++)
            {
                DAVtq actualVtq = target.ReadItem(serverDescriptor, "Simulation.Random");

                Assert.IsNotNull(actualVtq);
                Assert.IsTrue(actualVtq.Quality.IsGood);
                Assert.IsTrue(actualVtq.Quality.StatusBitField == DAStatusChoice.GoodNonspecific);
                Assert.IsTrue(actualVtq.Quality.LimitBitField == DALimitChoice.LimitOk);
                Assert.IsTrue(actualVtq.HasValue);
                //... test more

                AssertUtilities.WriteLine(actualVtq.Value);

                if (i % 10 == 0)
                {
                    delay = 100 + (int)(500.0 * random.NextDouble());
                    AssertUtilities.WriteLine(String.Format(CultureInfo.InvariantCulture, "Delay: {0}", delay));
                }
                Thread.Sleep(delay);
            }
        }

        // ReSharper restore InconsistentNaming
        #endregion

    }
}
