﻿using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.LiveMapping;
using OpcLabs.EasyOpcLiveTest.Legacy.Environment.OpcKitServer;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.LiveMapping._Testers
{
    class SimpleTester
    {
        private class TestRecord
        {
            public int Count { get; set; }
        }

        private class MyRootNode : RootNode
        {
            public MyRootNode([NotNull] TestRecord testRecord)
            {
                Simulation = new MySimulationNode(testRecord);
            }
        }

        private class MySimulationNode : SimulationNode
        {
            public MySimulationNode([NotNull] TestRecord testRecord)
            {
                TestRecord = testRecord;
            }

            [DARead(DataSource = DADataSource.Cache)]
            public override int IncrementingValue
            {
                set
                {
                    TestRecord.Count++;
                    AssertUtilities.WriteLine(value);
                }
            }

            [NotNull]
            // ReSharper disable MemberCanBePrivate.Local
            public TestRecord TestRecord { get; }
            // ReSharper restore MemberCanBePrivate.Local
        }

        public void SubscribeTest()
        {
            const int timeout = 10 * 1000;
            const int period = 1000;

            var testRecord = new TestRecord();

            var clientMapper = new DAClientMapper();
            var testTarget = new MyRootNode(testRecord);

            clientMapper.Map(testTarget, new DAMappingContext { ServerDescriptor = LegacyTestDefs.ProgIds.KitServer, GroupParameters = 100 });

            clientMapper.Subscribe(true);
            //mapper.Subscribe(mapper.Mappings.Where(mapping => mapping.BelongsTo(testTarget)));
            AssertUtilities.WaitingPhase(timeout);
            clientMapper.Subscribe(false);

            Assert.IsTrue(testRecord.Count >= timeout/period/2);
        }

        public void ReadTest()
        {
            var testRecord = new TestRecord();

            var clientMapper = new DAClientMapper();
            var testTarget = new MyRootNode(testRecord);

            clientMapper.Map(testTarget, new DAMappingContext { ServerDescriptor = LegacyTestDefs.ProgIds.KitServer });

            const int value = 827364873;

            new EasyDAClient().WriteItemValue(LegacyTestDefs.ProgIds.KitServer, LegacyTestDefs.DataAccess.ItemIds.Register_I4, value);

            testTarget.Simulation.Register_I4 = 999999999;
            clientMapper.Read();
            // TODO:
            //mapper.Read(from mapping in mapper.Mappings where mapping.GetTargetObject() == testTarget.Simulation select mapping);
            //mapper.Read(from mapping in mapper.Mappings where mapping.BelongsTo(testTarget.Simulation) select mapping);
            Assert.AreEqual(value, testTarget.Simulation.Register_I4);
        }
    }
}
