﻿using System;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Windows.Threading;

namespace OpcLabs.EasyOpcLiveTest.Legacy
{

    public class MultithreadingObservableCollection<T> : ObservableCollection<T>
    {
        public override event NotifyCollectionChangedEventHandler CollectionChanged;

        protected override void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
        {
            NotifyCollectionChangedEventHandler collectionChanged = /*this.*/CollectionChanged;
            if (collectionChanged != null)
                foreach (var @delegate in collectionChanged.GetInvocationList())
                {
                    var eventHandler = (NotifyCollectionChangedEventHandler) @delegate;
                    Debug.Assert(eventHandler != null);
                    if (eventHandler.Target is DispatcherObject dispObj)
                    {
                        Dispatcher dispatcher = dispObj.Dispatcher;
                        if (dispatcher != null && !dispatcher.CheckAccess())
                        {
                            dispatcher.BeginInvoke(
                                (Action)(() =>
                                    eventHandler.Invoke(this,
                                        new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset))),
                                DispatcherPriority.DataBind);
                            continue;
                        }
                    }
                    eventHandler.Invoke(this, e);
                }
        }
    }
}
