﻿using System;
using System.Threading;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Diagnostics;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace;
using OpcLabs.EasyOpcLiveTest.UA.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.UA
{
    class MultiThreadedReadTester
    {
        const int ThreadCount = 5;

        public MultiThreadedReadTester([NotNull] UAEndpointDescriptor endpointDescriptor)
        {
            EndpointDescriptor = endpointDescriptor;
        }

        [NotNull]
        public UAEndpointDescriptor EndpointDescriptor { get; }

        public void Test()
        {
            TestEnvironment.PrecheckEndpoint(EndpointDescriptor);

            _client.Write(EndpointDescriptor, _nodeId, _writeAttributeData, TypeCode.Int32);

            var threads = new Thread[ThreadCount];
            for (int i = 0; i < ThreadCount; i++)
                threads[i] = new Thread(ReadThread);

            for (int i = 0; i < ThreadCount; i++)
            {
                Check.Assert(threads[i] != null);
                threads[i].Start();
            }

            for (int i = 0; i < ThreadCount; i++)
            {
                Check.Assert(threads[i] != null);
                threads[i].Join();
            }
        }

        private void ReadThread()
        {
            AssertUtilities.WriteLine("thread begin");

            UAAttributeData attributeData = _client.Read(EndpointDescriptor, _nodeId);

            Assert.IsNotNull(attributeData);
            Assert.AreEqual(_writeAttributeData.StatusCode, attributeData.StatusCode);
            Assert.AreEqual(_writeAttributeData.Value, attributeData.Value);
        }

        [NotNull]
        private LegacyUATestEnvironment TestEnvironment => LegacyUATestEnvironment.Current;

        #region Private Fields

        [NotNull]
        private readonly EasyUAClient _client = new EasyUAClient();
        [NotNull]
        private readonly UANodeId _nodeId = OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_UserScalar_Int32Value;
        [NotNull]
        private readonly UAAttributeData _writeAttributeData = new UAAttributeData((object)1052);

        #endregion
    }
}
