﻿using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.ComponentModel.Linking;
using OpcLabs.BaseLib.Diagnostics;
using OpcLabs.BaseLib.Extensions.Internal;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.LiveMapping;
using OpcLabs.EasyOpcLiveTest.UA.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.UA.LiveMapping
{
    class TypelessMappingTester
    {
        private class TypelessMappingTestClass
        {
            public Int32 Value { get; set; }
        }

        public void Test()
        {
            TestSection.Arrange();

            UAEndpointDescriptor endpointDescriptor = UATestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.OpcTcp;
            using (TestEnvironment.RequestLockFor(endpointDescriptor))
            {
                TestEnvironment.PrecheckEndpoint(endpointDescriptor);

                var mapper = new UAClientMapper();
                var target = new TypelessMappingTestClass();

                var memberInfo = target.GetType().GetSingleMember("Value");
                Check.Assert(memberInfo != null);

                const string nodeIdExpandedText =
                    OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_UserScalar_Int32Value;

                mapper.DefineMapping(
                    new UAClientDataMappingSource(endpointDescriptor, nodeIdExpandedText,
                        UAAttributeId.Value, UAIndexRangeList.Empty, UAReadParameters.CacheMaximumAge),
                    new UAClientDataMapping(typeof(Int32)),
                    new ObjectMemberLinkingTarget(target.GetType(), target, memberInfo));

                const int value = 824487;
                new EasyUAClient().WriteValue(endpointDescriptor, nodeIdExpandedText, value);

                target.Value = 999999999;

                TestSection.Act();
                mapper.Read();

                TestSection.Assert();

                Assert.AreEqual(value, target.Value);
            }
        }

        [NotNull]
        private UATestEnvironment TestEnvironment => UATestEnvironment.Current;
    }
}
