﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
#if (NETFRAMEWORK)
using System.Net.Sockets;
#endif
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Collections.Generic.Extensions;
using OpcLabs.BaseLib.Collections.Specialized;
using OpcLabs.BaseLib.Utilities;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.Discovery;
using OpcLabs.EasyOpc.UA.OperationModel;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.UA
{
    partial class EasyUAClientTest
    {
#region Static Private Methods

        static private void DiscoveryElementCollectionAsserts([NotNull] UADiscoveryElementCollection discoveryElementCollection)
        {
            AssertUtilities.WriteLine();
            AssertUtilities.WriteLine($"discoveryElementCollection.Count: {discoveryElementCollection.Count}");

            foreach (UADiscoveryElement discoveryElement in discoveryElementCollection)
            {
                AssertUtilities.WriteLine();
                AssertUtilities.WriteLine(discoveryElement);
                if (discoveryElement == null)
                    continue;

                AssertUtilities.WriteLine($"ServerName: {discoveryElement.ServerName}");
                AssertUtilities.WriteLine($"DiscoveryUriString: {discoveryElement.DiscoveryUriString}");
                AssertUtilities.WriteLine($"ServerCapabilities: {discoveryElement.ServerCapabilities}");
            }

            AssertUtilities.WriteLine();

            CollectionAssert.AllItemsAreNotNull(discoveryElementCollection);
            CollectionAssert.AllItemsAreUnique(discoveryElementCollection);

            // IMPROVE: assert more
        }

#endregion



#region Test Methods

        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void DiscoverLocalServersTest()
        {
            TestSection.Arrange();
            string discoveryHost = UATestDefs.HostNames.OpcUATest;
            TestEnvironment.PrecheckLds(discoveryHost);

            var target = new EasyUAClient();

            TestSection.Act();
            UADiscoveryElementCollection discoveryElementCollection = target.DiscoverLocalServers(discoveryHost);

            TestSection.Assert();

            Assert.IsNotNull(discoveryElementCollection);

            DiscoveryElementCollectionAsserts(discoveryElementCollection);

            AssertUtilities.WriteLine();
            var knownEndpointDescriptors = new[]
            {
                UATestEnvironment.OpcFoundation_UAAlarmConditionServer.EndpointDescriptors.Http,
                UATestEnvironment.OpcFoundation_UAAlarmConditionServer.EndpointDescriptors.OpcTcp,
                UATestEnvironment.OpcFoundation_UAReferenceServer.EndpointDescriptors.Http,
                UATestEnvironment.OpcFoundation_UAReferenceServer.EndpointDescriptors.OpcTcp,
                UATestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.Http,
                UATestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.Https,
                UATestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.OpcTcp,
                UATestEnvironment.Softing_OpcUaNetDemoServer.EndpointDescriptors.Http,
                UATestEnvironment.Softing_OpcUaNetDemoServer.EndpointDescriptors.Https,
                UATestEnvironment.Softing_OpcUaNetDemoServer.EndpointDescriptors.OpcTcp,
                //UATestEnvironment.UnifiedAutomation_UaCppServer.EndpointDescriptors.OpcTcp,   // does not currently register with LDS
            };
            IList<UAEndpointDescriptor> connectableEndpointList =
                // ReSharper disable once AssignNullToNotNullAttribute
                knownEndpointDescriptors.Where(descriptor => TestEnvironment.CanConnectEndpoint(descriptor)).ToList();
            AssertUtilities.WriteLine("Connectable endpoints were:");
            connectableEndpointList.ForEach(AssertUtilities.WriteLine);

            AssertUtilities.WriteLine();
            int minimumCount = connectableEndpointList.Count;
            AssertUtilities.WriteLine($"minimumCount: {minimumCount}");

            Assert.IsTrue(discoveryElementCollection.Count >= minimumCount, "Not enough discovery elements.");
            if (discoveryElementCollection.Count == 0)
                Assert.Inconclusive();
            // IMPROVE: assert more
        }

        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void DiscoverLocalServersTest_PacketsDropped()
        {
            TestSection.Arrange();

            const string discoveryHost = UATestDefs.HostNames.OpcUATest;
            string uriTemplateString1 = UriTemplateUtilities.FormatUriTemplateString(UASchemes.OpcTcp, 4841, "UADiscovery");
            string uriTemplateString2 = UriTemplateUtilities.FormatUriTemplateString(UASchemes.Http, 52602, "UADiscovery");

            var target = new EasyUAClient { Isolated = true };
            target.IsolatedParameters.HostParameters.DiscoveryUriTemplateStrings.Clear();
            target.IsolatedParameters.HostParameters.DiscoveryUriTemplateStrings.Add(uriTemplateString1);
            target.IsolatedParameters.HostParameters.DiscoveryUriTemplateStrings.Add(uriTemplateString2);

            if (TestEnvironment.TestEndpoint(new UAEndpointDescriptor { Host = discoveryHost, Port = 4841}) != -1)
                Assert.Inconclusive("Failed pre-check: No timeout, port 4841.");
            if (TestEnvironment.TestEndpoint(new UAEndpointDescriptor { Host = discoveryHost, Port = 52602}) != -1)
                Assert.Inconclusive("Failed pre-check: No timeout, port 52602.");

            try
            {
                TestSection.Act();
                try
                {
                    target.DiscoverLocalServers(discoveryHost);
                }
                finally
                {
                    TestSection.Assert();
                }
            }
            catch (UAException uaException)
            {
                AssertUtilities.WriteLine($"uaException: {uaException}");
                Assert.IsTrue(
                    (unchecked((int)new UAStatusCode(UACodeBits.BadNotConnected, 0).InternalValue) == uaException.ErrorCode) ||
                    (unchecked((int)new UAStatusCode(UACodeBits.BadRequestTimeout, 0).InternalValue) == uaException.ErrorCode));
                // IMPROVE: test the message, etc.
                Exception innerException = uaException.InnerException;
                Assert.IsNotNull(innerException);
                AssertUtilities.WriteLine($"innerException: {innerException}");
                var serviceException = innerException as UAServiceException;
                Assert.IsNotNull(serviceException);
                // IMPROVE: test the contents of serviceException
                return;
            }
            Assert.Fail("UAException not thrown.");
        }

        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void DiscoverLocalServersTest_UnknownDiscoveryHost()
        {
            TestSection.Arrange();
            var target = new EasyUAClient();
            try
            {
                TestSection.Act();
                try
                {
                    target.DiscoverLocalServers("unknown.host.3A494877-A49C-4A51-B827-F1D8F5D03DF3");
                }
                finally
                {
                    TestSection.Assert();
                }
            }
            catch (UAException uaException)
            {
                AssertUtilities.WriteLine($"uaException: {uaException}");
#if (NETFRAMEWORK)
                Assert.AreEqual(-1, uaException.ErrorCode);
#else
                Assert.AreEqual(unchecked((int)0x808A0000), uaException.ErrorCode);
#endif
                // IMPROVE: test the message, etc.
                Exception innerException = uaException.InnerException;
                AssertUtilities.WriteLine($"innerException: {innerException}");
                AssertUtilities.WriteLine(innerException);
#if (NETFRAMEWORK)
                var socketException = innerException as SocketException;
                // ReSharper disable once UsePatternMatching
                var endpointNotFoundException = innerException as EndpointNotFoundException;
                Assert.IsTrue((socketException != null) || (endpointNotFoundException != null));
                // IMPROVE: test the contents of inner extension
#else
                var serviceException = innerException as UAServiceException;
                Assert.IsNotNull(serviceException);
                StringAssert.Contains(serviceException.Message, "BadNotConnected");
#endif
                return;
            }
            Assert.Fail("UAException not thrown.");
        }

        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void DiscoverNetworkServersTest()
        {
            TestSection.Arrange();
            var target = new EasyUAClient();
            DiscoverNetworkServersTest(target);
        }

        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void DiscoverNetworkServersTest_FilterOutEverything()
        {
            TestSection.Arrange();
            string discoveryHost = UATestDefs.HostNames.OpcUATest;
            TestEnvironment.PrecheckLds(discoveryHost);

            var target = new EasyUAClient();

            TestSection.Act();
            UADiscoveryElementCollection discoveryElementCollection = target.DiscoverNetworkServers(new[] { "X-UNKNOWN" }, discoveryHost);

            TestSection.Assert();

            Assert.IsNotNull(discoveryElementCollection);
            DiscoveryElementCollectionAsserts(discoveryElementCollection);

            Assert.AreEqual(0, discoveryElementCollection.Count);
        }

        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void DiscoverNetworkServersTest_OpcTcp()
        {
            TestSection.Arrange();
            var target = new EasyUAClient { Isolated = true };
            target.IsolatedParameters.HostParameters.DiscoveryUriTemplateStrings = new StringCollection(
                new[] { UriTemplateUtilities.FormatUriTemplateString(UASchemes.OpcTcp, UAPorts.OpcTcp, "") });
            DiscoverNetworkServersTest(target);
        }

        [TestMethod]
        public void DiscoverWithLocalEndpointDiscoveryQueryTest()
        {
            TestSection.Arrange();
            var endpointDescriptors = new[]
            {
                UATestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.Http,
                UATestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.Https,
                UATestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.OpcTcp
            };
            endpointDescriptors.ForEach(TestEnvironment.PrecheckEndpoint);

            var target = new EasyUAClient();
            // ReSharper disable once PossibleNullReferenceException
            IEnumerable<string> discoveryUriStrings = endpointDescriptors.Select(descriptor => descriptor.UrlString);
            var localEndpointDiscoveryQuery = new UALocalEndpointDiscoveryQuery(discoveryUriStrings);

            TestSection.Act();
            UADiscoveryElementCollection discoveryElementCollection = target.Discover(localEndpointDiscoveryQuery);

            TestSection.Assert();

            Assert.IsNotNull(discoveryElementCollection);
            DiscoveryElementCollectionAsserts(discoveryElementCollection);

            int count = discoveryElementCollection.Count;
            Assert.AreEqual(8, count);
            // IMPROVE: assert more
        }

#endregion



#region Private Methods

        private void DiscoverNetworkServersTest([NotNull] EasyUAClient target)
        {
            TestSection.Arrange();
            string discoveryHost = UATestDefs.HostNames.OpcUATest;
            TestEnvironment.PrecheckLds(discoveryHost);

            TestSection.Act();
            UADiscoveryElementCollection discoveryElementCollection = target.DiscoverNetworkServers(discoveryHost);

            TestSection.Assert();

            Assert.IsNotNull(discoveryElementCollection);
            DiscoveryElementCollectionAsserts(discoveryElementCollection);

            Assert.IsTrue(discoveryElementCollection.Count >= 2, "Not enough application elements.");
            // IMPROVE: assert more
        }

#endregion
    }
}
