﻿
using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Diagnostics;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.OperationModel;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcTest.DataAccess.OperationModel
{
    public partial class DAVtqResultTest
    {
        [TestCategory("BinarySerialization")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BinarySerializationTest0()
        {
            Tests.BinarySerializationTest(CreateSerializationTarget0());
        }

        [TestCategory("BinarySerialization")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BinarySerializationTest2()
        {
            Tests.BinarySerializationTest(CreateSerializationTarget2());
        }

        [TestCategory("BinarySerialization")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BinarySerializationTest3()
        {
            Tests.BinarySerializationTest(CreateSerializationTarget3(), 
                (target, actual) =>
                    {
                        // ReSharper disable PossibleNullReferenceException
                        Check.Assert(target.Vtq != null);

                        Assert.AreEqual((string)(target.State), (string)(actual.State));
                        Assert.IsNotNull(actual.Vtq);
                        Assert.AreEqual((string)(target.Vtq.Value), (string)(actual.Vtq.Value));
                        // ReSharper restore PossibleNullReferenceException
                    });
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("XmlSerialization")]
        [TestMethod]
        public void XmlSerializationTest0()
        {
            Tests.XmlSerializationTest(CreateSerializationTarget0());
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("XmlSerialization")]
        [TestMethod]
        public void XmlSerializationTest2()
        {
            Tests.XmlSerializationTest(CreateSerializationTarget2());
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("XmlSerialization")]
        [TestMethod]
        public void XmlSerializationTest3()
        {
            Tests.XmlSerializationTest(CreateSerializationTarget3(), 
                (target, actual) =>
                    {
                        // ReSharper disable PossibleNullReferenceException
                        Check.Assert(target.Vtq != null);

                        Assert.AreEqual((string)(target.State), (string)(actual.State));
                        Assert.IsNotNull(actual.Vtq);
                        Assert.AreEqual((string)(target.Vtq.Value), (string)(actual.Vtq.Value));
                        // ReSharper restore PossibleNullReferenceException
                    });
        }

        [NotNull]
        private DAVtqResult CreateSerializationTarget0()
        {
            return new DAVtqResult();
        }

        [NotNull]
        private DAVtqResult CreateSerializationTarget2()
        {
            return new DAVtqResult(null, new DAVtq(null, DateTime.MinValue, DAQualities.GoodNonspecific));
        }

        [NotNull]
        private DAVtqResult CreateSerializationTarget3()
        {
            return new DAVtqResult("TestState", new DAVtq("TestValue", DateTime.MinValue, DAQualities.GoodNonspecific));
        }
    }
}
