﻿using System.Runtime.InteropServices;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Diagnostics;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.OperationModel;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcTest.DataAccess
{
    [TestClass]
    public class EasyDAItemChangedEventArgsTest
    {
        [TestCategory("BinarySerialization")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BinarySerializationTest0()
        {
            Tests.BinarySerializationTest(CreateSerializationTarget0());
        }

        [TestCategory("BinarySerialization")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BinarySerializationTest1()
        {
            Tests.BinarySerializationTest(CreateSerializationTarget1(), SerializationAssertion1);
        }

        [TestCategory("BinarySerialization")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BinarySerializationTest2()
        {
            Tests.BinarySerializationTest(CreateSerializationTarget2(), SerializationAssertion2);
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("XmlSerialization")]
        [TestMethod]
        public void XmlSerializationTest0()
        {
            Tests.XmlSerializationTest(CreateSerializationTarget0());
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("XmlSerialization")]
        [TestMethod]
        public void XmlSerializationTest1()
        {
            Tests.XmlSerializationTest(CreateSerializationTarget1(), SerializationAssertion1);
        }

        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("XmlSerialization")]
        [TestMethod]
        public void XmlSerializationTest2()
        {
            Tests.XmlSerializationTest(CreateSerializationTarget2(), SerializationAssertion2);
        }

        [NotNull]
        public EasyDAItemChangedEventArgs CreateSerializationTarget0()
        {
            return new EasyDAItemChangedEventArgs();
        }

        [NotNull]
        public EasyDAItemChangedEventArgs CreateSerializationTarget1()
        {
            return new EasyDAItemChangedEventArgs
            {
                ErrorCode = 2222,
                Exception = new COMException("TestMessage"),
                Arguments = 
                {
                    GroupParameters = new DAGroupParameters(4444),
                    ItemDescriptor = new DAItemDescriptor("TestItemId"),
                    ServerDescriptor = new ServerDescriptor("TestServerClass"), 
                },
                Vtq = new DAVtq(null, DAQualities.GoodNonspecific)
            };
        }

        [NotNull]
        public EasyDAItemChangedEventArgs CreateSerializationTarget2()
        {
            return new EasyDAItemChangedEventArgs
            {
                Arguments = { State = "TestState", },
                Vtq = new DAVtq("TestValue")
            };
        }

        public void SerializationAssertion1(EasyDAItemChangedEventArgs target, EasyDAItemChangedEventArgs actual)
        {
            Assert.AreEqual(target, actual);
        }

        public void SerializationAssertion2(EasyDAItemChangedEventArgs target, EasyDAItemChangedEventArgs actual)
        {
            Check.Assert(target != null);
            Check.Assert(target.Vtq != null);

            Assert.IsNotNull(actual);
            Assert.AreEqual((string)(target.Arguments.State), (string)(actual.Arguments.State));
            DAVtq vtq = actual.Vtq;
            Assert.IsNotNull(vtq);
            Assert.AreEqual((string)(target.Vtq.Value), (string)(vtq.Value));
        }
    }
}
